package edu.lawrence.auction.interfaces;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import edu.lawrence.auction.entities.Auction;
import edu.lawrence.auction.entities.Bid;
import edu.lawrence.auction.interfaces.dtos.AuctionDTO;
import edu.lawrence.auction.interfaces.dtos.BidDTO;
import edu.lawrence.auction.services.AuctionService;


@RestController
@RequestMapping("/auctions")
@CrossOrigin(origins = "*")
public class AuctionController {
	private AuctionService auctionService;

    public AuctionController(AuctionService auctionService) {
        this.auctionService = auctionService;
    }
    
	@PostMapping
    public ResponseEntity<String> save(@RequestBody AuctionDTO auction) {
		String key = auctionService.save(auction);
        if (key.equals("Bid Id")) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Bad Seller Id");
        }
        return ResponseEntity.ok().body(key);
	}
	
	@PostMapping("/{id}/bids")
    public void saveBid(@PathVariable UUID id,@RequestBody BidDTO bid) {
		auctionService.saveBid(id, bid);
	}
	
	@GetMapping("/{id}/bids")
    public ResponseEntity<List<BidDTO>> findBidsForAuction(@PathVariable UUID id) {
		Auction auction = auctionService.findById(id);
		List<BidDTO> results = new ArrayList<BidDTO>();
		for(Bid b : auction.getBids()) {
			results.add(new BidDTO(b));
		}
		return ResponseEntity.ok().body(results);
	}
	
	@GetMapping("/{id}")
	public ResponseEntity<AuctionDTO> findById(@PathVariable UUID id) {
		Auction auction = auctionService.findById(id);
		AuctionDTO result = new AuctionDTO(auction);
		return ResponseEntity.ok().body(result);
	}
	
	@GetMapping(params= {"tag"})
	public ResponseEntity<List<AuctionDTO>> findActiveAuctionsByTag(@RequestParam(value = "tag") String tag) {
		List<Auction> auctions = auctionService.findActiveByTag(tag);
		List<AuctionDTO> results = new ArrayList<AuctionDTO>();
		for(Auction a : auctions) {
			results.add(new AuctionDTO(a));
		}
		return ResponseEntity.ok().body(results);
	}
	
	@GetMapping
	public ResponseEntity<List<AuctionDTO>> findActiveAuctions() {
		List<Auction> auctions = auctionService.findActive();
		List<AuctionDTO> results = new ArrayList<AuctionDTO>();
		for(Auction a : auctions) {
			results.add(new AuctionDTO(a));
		}
		return ResponseEntity.ok().body(results);
	}
}
