package edu.lawrence.auction.services;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import edu.lawrence.auction.entities.Auction;
import edu.lawrence.auction.entities.Profile;
import edu.lawrence.auction.entities.Purchase;
import edu.lawrence.auction.entities.Shipping;
import edu.lawrence.auction.entities.User;
import edu.lawrence.auction.interfaces.dtos.Bill;
import edu.lawrence.auction.interfaces.dtos.Offer;
import edu.lawrence.auction.interfaces.dtos.ProfileDTO;
import edu.lawrence.auction.interfaces.dtos.Sale;
import edu.lawrence.auction.interfaces.dtos.ShippingDTO;
import edu.lawrence.auction.interfaces.dtos.UserDTO;
import edu.lawrence.auction.repositories.ProfileRepository;
import edu.lawrence.auction.repositories.PurchaseRepository;
import edu.lawrence.auction.repositories.ShippingRepository;
import edu.lawrence.auction.repositories.UserRepository;

@Service
public class UserService {
	@Autowired 
    PasswordService passwordService;
	
	@Autowired
	UserRepository userRepository;
	
	@Autowired
	ShippingRepository shippingRepository;
	
	@Autowired
	ProfileRepository profileRepository;
	
	@Autowired 
	PurchaseRepository purchaseRepository;
	
	public String save(UserDTO user) {
		List<User> existing = userRepository.findByName(user.getName());
		if(existing.size() > 0)
			return "Duplicate";
		
		User newUser = new User();
		newUser.setName(user.getName());
		String hash = passwordService.hashPassword(user.getPassword());
	    newUser.setPassword(hash);
		userRepository.save(newUser);
		return newUser.getUserid().toString();
	}
	
	public User findByNameAndPassword(String name,String password) {
		List<User> existing = userRepository.findByName(name);
		if(existing.size() != 1)
			return null;
		User u = existing.get(0);
		if(passwordService.verifyHash(password, u.getPassword())) {
        	u.setPassword("Undisclosed");
        } else {
        	u = null;
        }
        return u;	
	}
	
	public String saveProfile(UUID userid,ProfileDTO profile) {
		Optional<User> maybeUser = userRepository.findById(userid);
		if(!maybeUser.isPresent())
			return "Bad Id";
		
		User user = maybeUser.get();
		if(user.getProfile() != null)
			return "Duplicate";
		
		Shipping newShipping = new Shipping(profile.getShipping());
		newShipping.setUser(user);
		newShipping = shippingRepository.save(newShipping);
		
		Profile newProfile = new Profile(profile);
		newProfile.setUser(user);
		newProfile.setShipping(newShipping);
		profileRepository.save(newProfile);
		
		return "Created";
	}
	
	public Profile findProfile(UUID userid) {
		Optional<User> maybeUser = userRepository.findById(userid);
		if(!maybeUser.isPresent())
			return null;
		
		return maybeUser.get().getProfile();
	}
	
	public String saveShipping(UUID userid,ShippingDTO shipping) {
		Optional<User> maybeUser = userRepository.findById(userid);
		if(!maybeUser.isPresent())
			return "Bad Id";
		
		User user = maybeUser.get();
		Shipping newShipping = new Shipping(shipping);
		newShipping.setUser(user);
		shippingRepository.save(newShipping);
		return "Created";
	}
	
	public List<Shipping> findShipping(UUID userid) {
		Optional<User> maybeUser = userRepository.findById(userid);
		if(!maybeUser.isPresent())
			return new ArrayList<Shipping>();
		return maybeUser.get().getShipping();
	}
	
	public List<Auction> findAuctions(UUID userid) {
		Optional<User> maybeUser = userRepository.findById(userid);
		if(!maybeUser.isPresent())
			return new ArrayList<Auction>();
		return maybeUser.get().getAuctions();
	}
	
	public List<Offer> findOffers(UUID userid) {
		List<Purchase> purchases = purchaseRepository.findOffers(userid);
		
		List<Offer> offers = new ArrayList<Offer>();
		for(Purchase p : purchases) {
			offers.add(new Offer(p));
		}
		return offers;
	}
	
	public List<Sale> findSales(UUID userid) {
		List<Purchase> purchases = purchaseRepository.findSales(userid);
		
		List<Sale> sales = new ArrayList<Sale>();
		for(Purchase p : purchases) {
			sales.add(new Sale(p));
		}
		return sales;
	}
	
	public List<Bill> findBills(UUID userid) {
		List<Purchase> purchases = purchaseRepository.findBills(userid);
		
		List<Bill> bills = new ArrayList<Bill>();
		for(Purchase p : purchases) {
			bills.add(new Bill(p));
		}
		return bills;
	}
	
	public List<Bill> findSoldBills(UUID userid) {
		List<Purchase> purchases = purchaseRepository.findSoldBills(userid);
		
		List<Bill> bills = new ArrayList<Bill>();
		for(Purchase p : purchases) {
			bills.add(new Bill(p));
		}
		return bills;
	}
	
	public List<Bill> findShippedBills(UUID userid) {
		List<Purchase> purchases = purchaseRepository.findShippedBills(userid);
		
		List<Bill> bills = new ArrayList<Bill>();
		for(Purchase p : purchases) {
			bills.add(new Bill(p));
		}
		return bills;
	}
}
